﻿using GarmentFactoryContracts.BindingModels;
using GarmentFactoryContracts.ViewModels;
using GarmentFactoryDataModels.Enums;
using GarmentFactoryDataModels.Models;
using System;
using System.Collections.Generic;
using System.ComponentModel.DataAnnotations;
using System.Linq;
using System.Runtime.Serialization;
using System.Text;
using System.Threading.Tasks;

namespace GarmentFactoryDatabaseImplement.Models
{
	[DataContract]
	public class Order : IOrderModel
    {
		[DataMember]
		public int Id { get; set; }
		[DataMember]
		[Required]
        public int TextileId { get; set; }
		[DataMember]
		public int? ImplementerId { get; private set; }
		[DataMember]
		[Required]
        public int ClientId { get; set; }
		[DataMember]
		[Required]
        public int Count { get; set; }
		[DataMember]
		[Required]
        public double Sum { get; set; }
		[DataMember]
		[Required]
        public OrderStatus Status { get; set; }
		[DataMember]
		[Required]
        public DateTime DateCreate { get; set; }
		[DataMember]
		public DateTime? DateImplement { get; set; }

        public virtual Textile Textile { get; set; }

        public virtual Client Client { get; set; }
        public virtual Implementer? Implementer { get; set; }

        public static Order? Create(OrderBindingModel? model)
        {
            if (model == null)
            {
                return null;
            }
            return new Order()
            {
                Id = model.Id,
                TextileId = model.TextileId,
                ClientId = model.ClientId,
                ImplementerId = model.ImplementerId,
                Count = model.Count,
                Sum = model.Sum,
                Status = model.Status,
                DateCreate = model.DateCreate,
                DateImplement = model.DateImplement
            };
        }

        public void Update(OrderBindingModel? model)
        {
            if (model == null)
            {
                return;
            }
            Status = model.Status;
            DateImplement = model.DateImplement;
            ImplementerId = model.ImplementerId;
        }

        public OrderViewModel GetViewModel => new()
        {
            Id = Id,
            TextileId = TextileId,
            ClientId = ClientId,
            ImplementerId = ImplementerId,
            Count = Count,
            Sum = Sum,
            Status = Status,
            DateCreate = DateCreate,
            DateImplement = DateImplement,
            ClientEmail = Client.Email ?? string.Empty,
            TextileName = Textile.TextileName ?? string.Empty,
            ClientFIO = Client.ClientFIO ?? string.Empty,
            ImplementerFIO = Implementer?.ImplementerFIO ?? string.Empty
        };
    }
}

