﻿using GarmentFactoryContracts.BindingModels;
using GarmentFactoryContracts.SearchModels;
using GarmentFactoryContracts.StoragesContracts;
using GarmentFactoryContracts.ViewModels;
using GarmentFactoryFileImplement.Models;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace GarmentFactoryFileImplement.Implements
{
    public class OrderStorage : IOrderStorage
    {
        private readonly DataFileSingleton source;

        public OrderStorage()
        {
            source = DataFileSingleton.GetInstance();
        }

        public List<OrderViewModel> GetFullList()
        {
            return source.Orders
                    .Select(x => AddIntelligence(x.GetViewModel))
                    .ToList();
        }

        public List<OrderViewModel> GetFilteredList(OrderSearchModel model)
        {
            if (model.Id.HasValue)
            {
                return source.Orders
                              .Where(x => x.Id == model.Id)
                              .Select(x => AddIntelligence(x.GetViewModel))
                              .ToList();
            }
            else if (model.DateFrom.HasValue && model.DateTo.HasValue)
            {
                return source.Orders
                              .Where(x => x.DateCreate >= model.DateFrom && x.DateCreate <= model.DateTo)
                              .Select(x => AddIntelligence(x.GetViewModel))
                              .ToList();
            }
            else if (model.ClientId.HasValue)
            {
                return source.Orders
                              .Where(x => x.ClientId == model.ClientId)
                              .Select(x => AddIntelligence(x.GetViewModel))
                              .ToList();
            }
            else if (model.ImplementerId.HasValue)
            {
                return source.Orders
                              .Where(x => x.ImplementerId == model.ImplementerId)
                              .Select(x => AddIntelligence(x.GetViewModel))
                              .ToList();
            }
            else if (model.Status != null)
            {
                return source.Orders
                              .Where(x => x.Status.Equals(model.Status))
                              .Select(x => AddIntelligence(x.GetViewModel))
                              .ToList();
            }
            return new();
        }
        public OrderViewModel? GetElement(OrderSearchModel model)
        {
            if (!model.Id.HasValue && !model.ImplementerId.HasValue)
            {
                return null;
            }
            var order = source.Orders.FirstOrDefault(x =>
                model.ImplementerId.HasValue && x.ImplementerId == model.ImplementerId && model.Status != null && x.Status.Equals(model.Status)
                || model.Status == null && model.ImplementerId.HasValue && x.ImplementerId == model.ImplementerId
                || model.Id.HasValue && x.Id == model.Id);
            return order?.GetViewModel != null ? AddIntelligence(order.GetViewModel) : null;
        }

        public OrderViewModel? Insert(OrderBindingModel model)
        {
            model.Id = source.Orders.Count > 0 ? source.Orders.Max(x => x.Id) + 1 : 1;
            var newOrder = Order.Create(model);
            if (newOrder == null)
            {
                return null;
            }
            source.Orders.Add(newOrder);
            source.SaveOrders();
            return AddIntelligence(newOrder.GetViewModel);
        }

        public OrderViewModel? Update(OrderBindingModel model)
        {
            var order = source.Orders.FirstOrDefault(x => x.Id == model.Id);
            if (order == null)
            {
                return null;
            }
            order.Update(model);
            source.SaveOrders();
            return AddIntelligence(order.GetViewModel);
        }

        public OrderViewModel? Delete(OrderBindingModel model)
        {
            var element = source.Orders.FirstOrDefault(x => x.Id == model.Id);
            if (element != null)
            {
                source.Orders.Remove(element);
                source.SaveOrders();
                return AddIntelligence(element.GetViewModel);
            }
            return null;
        }

        private OrderViewModel AddIntelligence(OrderViewModel model)
        {
            var selectedTextile = source.Textils.FirstOrDefault(x => x.Id == model.TextileId);
            model.TextileName = selectedTextile?.TextileName ?? string.Empty;
            var selectedClient = source.Clients.FirstOrDefault(x => x.Id == model.ClientId);
            model.ClientFIO = selectedClient?.ClientFIO ?? string.Empty;
            return model;
        }
    }
}
